<?php

namespace Drupal\idealyzer\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\idealyzer\Entity\SiteScannerResult;

class ReportController extends ControllerBase {
    protected $entityTypeManager;

    public function __construct(EntityTypeManagerInterface $entityTypeManager) {
        $this->entityTypeManager = $entityTypeManager;
    }

    public static function create(ContainerInterface $container) {
        return new static(
            $container->get('entity_type.manager')
        );
    }

    public function displayReport(SiteScannerResult $site_scanner_result) {
        if (!$site_scanner_result) {
            return [
                '#type' => 'markup',
                '#markup' => $this->t('No entity found.'),
            ];
        }

        // Log the retrieved entity data
        // \Drupal::logger('idealyzer')->debug('Retrieved entity data: <pre>' . print_r($site_scanner_result->toArray(), TRUE) . '</pre>');

        $build = [];

        if ($site_scanner_result->get('scan_date')->value) {
            // Extract scan info
            $build['scan_information'] = $this->buildScanInformation($site_scanner_result);

            // Build categorized tables
            $categorizedItems = $this->parseEntity($site_scanner_result);
            foreach ($categorizedItems as $category => $items) {
                $build[$category] = [
                    '#type' => 'details',
                    '#title' => $this->t($category),
                    '#open' => true,
                    'table' => [
                        '#type' => 'table',
                        '#header' => $this->getTableHeaders(),
                        '#rows' => $this->formatRowsForTable($items),
                        '#empty' => $this->t('No results for this category.'),
                    ],
                ];
            }
        } else {
            $build = [
                '#type' => 'markup',
                '#markup' => $this->t('No items found.'),
            ];
        }

        return $build;
    }

    private function buildScanInformation(SiteScannerResult $entity) {
        $scanTitle = $this->formatValue($entity->get('target_url_domain')->value ?? 'N/A');
        $scanDate = $this->formatValue($entity->get('scan_date')->value ?? 'N/A');
        $targetUrlDomain = $this->formatValue($entity->get('target_url_domain')->value ?? 'N/A');
        $completedScans = 0;
        $totalScans = 0;

        $scanStatuses = [
            'not_found_scan_status',
            'primary_scan_status',
            'robots_txt_scan_status',
            'sitemap_xml_scan_status',
            'dns_scan_status',
            'accessibility_scan_status',
            'performance_scan_status',
            'security_scan_status',
        ];

        foreach ($scanStatuses as $status) {
            if ($entity->hasField($status)) {
                $totalScans++;
                if ($entity->get($status)->value === 'completed') {
                    $completedScans++;
                }
            }
        }

        $overviewText = [
            $this->t('Scan Overview: @domain - @title', ['@domain' => $targetUrlDomain, '@title' => $scanTitle]),
            $this->t('Scan Date: @date', ['@date' => $scanDate]),
            $this->t('Completed Scans: @completed/@total', [
                '@completed' => $completedScans,
                '@total' => $totalScans,
            ]),
        ];

        return [
            '#type' => 'table',
            '#header' => [$overviewText[0], ''],
            '#rows' => [
                [
                    [
                        'data' => $overviewText[1],
                        'class' => 'scan-info-key',
                    ],
                    [
                        'data' => $overviewText[2],
                        'class' => 'scan-info-value',
                    ],
                ],
            ],
        ];
    }

    private function parseEntity(SiteScannerResult $entity) {
        $categories = [
            'Domain Information' => ['target_url_domain', 'final_url', 'final_url_status_code', 'ipv6', 'final_url_media_type', 'final_url_top_level_domain', 'target_url', 'target_url_top_level_domain', 'target_url_branch', 'target_url_agency_owner', 'target_url_bureau_owner', 'cms', 'source_list', 'public'],
            'Security' => ['https_enforced', 'hsts','third_party_service_domains', 'robots_txt_final_url', 'robots_txt_final_url_status_code', 'robots_txt_final_url_live', 'robots_txt_detected', 'robots_txt_final_url_filesize', 'sitemap_xml_final_url', 'sitemap_xml_final_url_status_code', 'sitemap_xml_final_url_live', 'sitemap_xml_detected', 'sitemap_xml_final_url_filesize', 'sitemap_xml_count', 'sitemap_xml_pdf_count'],
            'Content' => ['dap', 'dap_parameters', 'og_title', 'og_description', 'main_element_present', 'title', 'description', 'language', 'viewport_meta_tag', 'canonical_link', 'required_links_url', 'required_links_text'],
            'Performance' => ['largest_contentful_paint', 'cumulative_layout_shift', 'final_url_live', 'final_url_same_domain', 'final_url_same_website', 'target_url_404_test', 'target_url_redirects', 'hostname'],
            'USWDS' => ['uswds_usa_classes', 'uswds_string', 'uswds_inpage_css', 'uswds_favicon', 'uswds_string_in_css', 'uswds_favicon_in_css', 'uswds_publicsans_font', 'uswds_semantic_version', 'uswds_version', 'uswds_count']
        ];

        $categorizedItems = [];
        foreach ($categories as $category => $keys) {
            $categorizedItems[$category] = [];
        }

        foreach ($entity->toArray() as $key => $value) {
            foreach ($categories as $category => $keys) {
                if (in_array($key, $keys)) {
                    $categorizedItems[$category][] = [$key => $this->extractFieldValue($value)];
                }
            }
        }

        return $categorizedItems;
    }
    


    private function extractFieldValue($field_value) {

        if (is_array($field_value)) {
            // If the field is an array and contains 'value', extract it
            if (isset($field_value[0]['value'])) {
                return $field_value[0]['value'];
            } else {
                // Handle cases where the field might be a list of values (e.g., text lists)
                return implode(', ', array_map(function($item) {
                    return isset($item['value']) ? $item['value'] : 'N/A';
                }, $field_value));
            }
        }
        return $field_value;
    }
    


    private function formatValue($value) {
        if ($value === null) {
            return 'N/A';
        }
        if (is_array($value)) {
            return implode(', ', array_map([$this, 'formatValue'], $value));
        }
        return htmlspecialchars((string) $value, ENT_QUOTES, 'UTF-8');
    }

    private function getTableHeaders() {
        return ['Field', 'Value'];
    }

    private function determineStyle($value) {
        switch ($value) {
            case 'completed':
            case 'true':
                return ['icon' => 'check-circle', 'class' => 'pass'];
            case 'failed':
            case 'false':
                return ['icon' => 'times-circle', 'class' => 'fail'];
            case 'N/A':
            case null:
                return ['icon' => 'question-circle', 'class' => 'na'];
            default:
                return ['icon' => 'info-circle', 'class' => 'info'];
        }
    }

    private function formatRowsForTable($items) {
        $rows = [];
        foreach ($items as $item) {
            foreach ($item as $key => $value) {
                $formattedValue = is_array($value) ? implode(', ', array_map([$this, 'extractFieldValue'], $value)) : $value;
                $styleInfo = $this->determineStyle($formattedValue);
                $rows[] = [
                    [
                        'data' => [
                            '#type' => 'html_tag',
                            '#tag' => 'span',
                            '#value' => $key,
                            '#attributes' => ['class' => [$styleInfo['class']]]
                        ],
                    ],
                    [
                        'data' => [
                            '#markup' => $this->t('<i class="fa fa-:icon"></i> :value', [
                                ':icon' => $styleInfo['icon'], 
                                ':value' => htmlspecialchars($formattedValue, ENT_QUOTES, 'UTF-8')
                            ]),
                            '#allowed_tags' => ['i'],
                        ],
                    ],
                ];
            }
        }
        return $rows;
    }
    
}
