<?php

namespace Drupal\idealyzer\Controller;

use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\idealyzer\Entity\SiteScannerResult;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;


/**
 * Class SiteScannerResultController.
 */
class SiteScannerResultDownloadController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Constructs a new SiteScannerResultController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, FormBuilderInterface $form_builder) {
    $this->entityTypeManager = $entity_type_manager;
    $this->formBuilder = $form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('form_builder')
    );
  }

  /**
   * Displays the site scanner results list with the start scan form.
   *
   * @return array
   *   A render array.
   */
  public function listing() {
    $build['start_scan'] = $this->formBuilder->getForm('Drupal\idealyzer\Form\SiteScannerForm');

    $build['table'] = $this->entityTypeManager->getListBuilder('site_scanner_result')->render();

    return $build;
  }

  /**
   * Download the raw data as JSON.
   *
   * @param \Drupal\idealyzer\Entity\SiteScannerResult $site_scanner_result
   *   The site scanner result entity.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The JSON response.
   */
  public function downloadJson(SiteScannerResult $site_scanner_result) {
    $raw_data = $site_scanner_result->get('scan_data')->value;

    if ($raw_data === NULL) {
      \Drupal::logger('idealyzer')->error('No raw data found for entity ID: @id', ['@id' => $site_scanner_result->id()]);
      return new Response('No data found', 404);
    }

    $response = new Response();
    $response->setContent($raw_data);
    $response->headers->set('Content-Type', 'application/json');
    $response->headers->set('Content-Disposition', 'attachment; filename="site_scan.json"');

    return $response;
  }

  /**
   * Download CSV data for a specific Site Scanner Result entity.
   *
   * @param \Drupal\idealyzer\Entity\SiteScannerResult $site_scanner_result
   *   The Site Scanner Result entity.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The CSV response.
   */
  public function downloadCsv(SiteScannerResult $site_scanner_result) {
    $data = $site_scanner_result->toArray();

    // Convert data to CSV format
    $csv = $this->arrayToCsv($data);

    $response = new Response($csv);
    $response->headers->set('Content-Type', 'text/csv');
    $response->headers->set('Content-Disposition', 'attachment; filename="site_scan.csv"');
    return $response;
  }

  /**
   * Convert an array to CSV format.
   *
   * @param array $data
   *   The data array.
   *
   * @return string
   *   The CSV formatted string.
   */
  protected function arrayToCsv(array $data) {
    $output = fopen('php://output', 'w');
    ob_start();
    
    // Write the headers
    fputcsv($output, array_keys($data));

    // Write the data
    $this->writeCsvRow($output, $data);

    fclose($output);
    return ob_get_clean();
  }

  /**
   * Write a single row of data to the CSV.
   *
   * @param resource $output
   *   The output stream.
   * @param array $data
   *   The data to write.
   */
  protected function writeCsvRow($output, array $data) {
    foreach ($data as $key => $value) {
      if (is_array($value)) {
        // Flatten nested arrays
        $data[$key] = implode(', ', $value);
      } elseif (is_object($value)) {
        // Handle objects if necessary
        $data[$key] = json_encode($value);
      }
    }
    fputcsv($output, $data);
  }

  
}
