<?php

namespace Drupal\idealyzer\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Database\Database;

/**
 * Provides a Site Scan Filter form.
 */
class SiteScannerFilterForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'site_scan_filter_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Populate the dropdown options with unique target_url_agency_owner values from the database.
    $options = $this->getAgencyOwnerOptions();
    $Urloptions = $this->getTargetUrlOptions();
    $Statusoptions = $this->getStatusOptions();
    // Define the main container for the filters.
    $form['filters'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['filter-form-container']],
    ];

    // Define individual filter elements.
    $form['filters']['keyword'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Keyword Search'),
      '#size' => 20,
      '#maxlength' => 128,
      '#attributes' => ['class' => ['filter-form-inline']],
    ];
    
    $form['filters']['target_url_domain'] = [
      '#type' => 'select',
      '#title' => $this->t('Search by Domain'),
      '#options' => $Urloptions,
      '#empty_option' => $this->t('- Select an URL -'),
      '#attributes' => ['class' => ['filter-form-inline']],
    ];

    $form['filters']['agency_owner'] = [
      '#type' => 'select',
      '#title' => $this->t('Search by Agency'),
      '#options' => $options,
      '#empty_option' => $this->t('- Select an Agency -'),
      '#attributes' => ['class' => ['filter-form-inline']],
    ];
    $form['filters']['final_url_status_code'] = [
      '#type' => 'select',
      '#title' => $this->t('Response code'),
      '#options' => $Statusoptions,
      '#empty_option' => $this->t('- Select an Status -'),
      '#attributes' => ['class' => ['filter-form-inline']],
    ];
    // Define the actions container.
    $form['actions'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['filter-form-actions']],
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
      '#attributes' => ['class' => ['filter-form-inline']],
    ];

    $form['actions']['clear'] = [
      '#type' => 'submit',
      '#value' => $this->t('Clear'),
      '#submit' => ['::clearForm'],
      '#attributes' => ['class' => ['filter-form-inline']],
    ];

    return $form;
  }

  /**
   * Get unique agency owner options from the database.
   *
   * @return array
   *   An array of unique agency owner options.
   */
  protected function getAgencyOwnerOptions() {
    $connection = Database::getConnection();
    $query = $connection->select('site_scanner_results', 'ssr')
      ->fields('ssr', ['target_url_agency_owner'])
      ->distinct()
      ->orderBy('target_url_agency_owner');

    $result = $query->execute();
    $options = [];

    foreach ($result as $record) {
      $options[$record->target_url_agency_owner] = $record->target_url_agency_owner;
    }

    return $options;
  }

  protected function getTargetUrlOptions() {
    $connection = Database::getConnection();
    $query = $connection->select('site_scanner_results', 'ssr')
      ->fields('ssr', ['target_url_domain'])
      ->distinct()
      ->orderBy('target_url_domain');

    $result = $query->execute();
    $options = [];

    foreach ($result as $record) {
      $options[$record->target_url_domain] = $record->target_url_domain;
    }

    return $options;
  }

  protected function getStatusOptions() {
    $connection = Database::getConnection();
    $query = $connection->select('site_scanner_results', 'ssr')
      ->fields('ssr', ['final_url_status_code'])
      ->distinct()
      ->orderBy('final_url_status_code');

    $result = $query->execute();
    $options = [];

    foreach ($result as $record) {
      $options[$record->final_url_status_code] = $record->final_url_status_code;
    }

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $query = [];
    
    if ($keyword = $form_state->getValue('keyword')) {
      $query['keyword'] = $keyword;
    }
    
    if ($target_url_domain = $form_state->getValue('target_url_domain')) {
      $query['target_url_domain'] = $target_url_domain;
    }
    if ($agency_owner = $form_state->getValue('agency_owner')) {
      $query['agency_owner'] = $agency_owner;
    }
    if ($final_url_status_code = $form_state->getValue('final_url_status_code')) {
      $query['final_url_status_code'] = $final_url_status_code;
    }
    
    $form_state->setRedirect('idealyzer.site_scanner_result.collection', [], ['query' => $query]);
  }

  /**
   * Custom clear form submission handler.
   */
  public function clearForm(array &$form, FormStateInterface $form_state) {
    $form_state->setRedirect('idealyzer.site_scanner_result.collection');
  }
}
