<?php

namespace Drupal\idealyzer\Service;

use Drupal\Component\Serialization\Json;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\DependencyInjection\ContainerInterface; // Import ContainerInterface
use Drupal\idealyzer\Service\SiteScannerResultService; // Import SiteScannerResultService

class SiteScannerService {

  protected $httpClient;
  protected $configFactory;
  protected $resultService; 

  public function __construct(Client $httpClient, $configFactory, SiteScannerResultService $resultService) {
    $this->httpClient = $httpClient;
    $this->configFactory = $configFactory;
    $this->resultService = $resultService; // Initialize SiteScannerResultService
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('http_client'),
      $container->get('config.factory'),
      $container->get('idealyzer.site_scanner_result_service') // Get the SiteScannerResultService service
    );
  }

  /**
   * Retrieves the stored API key from configuration.
   *
   * @return string The API key.
   */
  protected function getApiKey() {
    // Retrieve the actual API key from the configuration.
    return $this->configFactory->get('idealyzer.settings')->get('api_key');
  }

  /**
   * Fetches and saves results from the Site Scanner API.
   */
  public function fetchAndSaveResults($url) {
    $apiKey = $this->getApiKey();
    $page = 1;
    $apiUrl = 'https://api.gsa.gov/technology/site-scanning/v1/websites';

    $has_more = true;

    while ($has_more) {
      try {
        $response = $this->httpClient->request('GET', $apiUrl, [
          'query' => [
            'target_url_domain' => $url,
            'api_key' => $apiKey,
            'page' => $page
          ]
        ]);

        $statusCode = $response->getStatusCode();
        $contents = $response->getBody()->getContents();

        if ($statusCode === 200) {
          $result = Json::decode($contents);
          //\Drupal::logger('idealyzer')->info('Requested URL: @url with response status: @status Data: @data', ['@url' => $url, '@status' => $statusCode, '@data' => print_r($result['meta']['totalPages'], TRUE)]);
          $this->saveResults($result);
          $has_more = $result['meta']['currentPage']<=$result['meta']['totalPages'];

          $page++;
        } else {
          \Drupal::logger('idealyzer')->error('API error with status @status: @response', ['@status' => $statusCode, '@response' => $contents]);
          return [
            'success' => false,
            'error' => t('API responded with status code: @status', ['@status' => $statusCode])
          ];
        }
      } catch (GuzzleException $e) {
        watchdog_exception('idealyzer', $e);
        return [
          'success' => false,
          'error' => $e->getMessage()
        ];
      }
    }
    return [
      'success' => true
    ];
  }

  /**
   * Save multiple SiteScannerResult entities from the API response.
   *
   * @param array $response
   *   The API response data.
   */
  public function saveResults(array $response) {
    $this->resultService->saveResults($response);
  }
}

