<?php

namespace Drupal\idealyzer;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Database\Database;
use Drupal\Core\Url;
use Drupal\Core\Database\Query\PagerSelectExtender;

/**
 * Defines a class to build a listing of Site Scanner Result entities.
 */
class SiteScannerResultListBuilder extends EntityListBuilder {

  /**
   * {@inheritdoc}
   */
  public function render() {

    // Calculate totals.
    $totals = $this->calculateTotals();

    $build['totals_accordion'] = [
      '#type' => 'details',
      '#title' => $this->t('Result Totals'),
      '#open' => TRUE,
      '#attributes' => ['class' => ['accordion']],
      'content' => [
        '#markup' => '<div class="totals-column">' .
                     '<strong>' . $this->t('Total Results:') . '</strong> ' . $totals['total'] . '</div>' .
                     '<div class="totals-column">' .
                     '<strong>' . $this->t('Unique Agencies:') . '</strong> ' . $totals['unique_agencies'] . '</div>' .
                     '<div class="totals-column">' .
                     '<strong>' . $this->t('DAP Usage:') . '</strong> ' . $totals['dap_percentage'] . '%</div>',
      ],
    ];

    // Render the table.
    $build['table'] = parent::render();
    // Attach the custom CSS library  
    $build['#attached']['library'][] = 'idealyzer/styles';
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['scan_date'] = $this->t('Scan Date');
    //$header['target_url_agency_owner'] = $this->t('Agency');
    $header['title'] = $this->t('Title');
    $header['final_url'] = $this->t('Final URL');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row['scan_date'] = $entity->get('scan_date')->value;
    //$row['target_url_agency_owner'] = $entity->get('target_url_agency_owner')->value;
    $row['title'] = $entity->get('title')->value;
    $row['final_url'] = $entity->get('final_url')->value;
    return $row + parent::buildRow($entity);
  }

  /**
   * Loads the entities with applied filters and pagination.
   *
   * @return array
   *   An array of loaded entities.
   */
  public function load() {
    $query = $this->getFilteredQuery();

    // Add pagination.
    $query = $query->extend(PagerSelectExtender::class)->limit(10);
    $results = $query->execute()->fetchAll();
    $entities = [];

    foreach ($results as $result) {
      $entities[] = $this->storage->load($result->id);
    }

    return $entities;
  }

  /**
   * Gets the filtered query without pagination.
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   *   The filtered query.
   */
  protected function getFilteredQuery() {
    $connection = Database::getConnection();
    $query = $connection->select('site_scanner_results', 'ssr')
      ->fields('ssr', ['id', 'scan_date', 'target_url_agency_owner', 'final_url', 'dap', 'final_url_status_code']); // Add other fields as needed.

    // Apply filters based on query parameters.
    $final_url = \Drupal::request()->query->get('final_url');
    if ($final_url) {
      $query->condition('ssr.final_url', '%' . $connection->escapeLike($final_url) . '%', 'LIKE');
    }

    $keyword = \Drupal::request()->query->get('keyword');
    if ($keyword) {
      $group = $query->orConditionGroup()
        ->condition('ssr.final_url', '%' . $connection->escapeLike($keyword) . '%', 'LIKE')
        ->condition('ssr.target_url_agency_owner', '%' . $connection->escapeLike($keyword) . '%', 'LIKE'); // Adjust fields to be searched.
      $query->condition($group);
    }

    $target_url_domain = \Drupal::request()->query->get('target_url_domain');
    if ($target_url_domain) {
      $query->condition('ssr.target_url_domain', $target_url_domain);
    }

    $agency_owner = \Drupal::request()->query->get('agency_owner');
    if ($agency_owner) {
      $query->condition('ssr.target_url_agency_owner', $agency_owner);
    }

    $final_url_status_code = \Drupal::request()->query->get('final_url_status_code');
    if ($final_url_status_code) {
      $query->condition('ssr.final_url_status_code', $final_url_status_code);
    }

    return $query;
  }

  /**
   * Calculates totals from the filtered list.
   *
   * @return array
   *   An array containing total results, unique agencies, and DAP usage percentage.
   */
  protected function calculateTotals() {
    $query = $this->getFilteredQuery();
    $results = $query->execute()->fetchAll();

    $total_results = count($results);
    $unique_agencies = count(array_unique(array_column($results, 'target_url_agency_owner')));

    // Calculate DAP usage percentage.
    $dap_count = count(array_filter(array_column($results, 'dap'), function($dap) {
      return $dap == 1; // Ensure this is checking for true values.
    }));
    $dap_percentage = $total_results > 0 ? round(($dap_count / $total_results) * 100, 2) : 0;

    return [
      'total' => $total_results,
      'unique_agencies' => $unique_agencies,
      'dap_percentage' => $dap_percentage,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $operations = [];

    if ($entity->access('view')) {
      $operations['view'] = [
        'title' => t('View'),
        'weight' => -10,
        'url' => Url::fromRoute('entity.site_scanner_result.canonical', ['site_scanner_result' => $entity->id()]),
      ];
    }

    if ($entity->access('view')) {
      $operations['download_json'] = [
        'title' => t('Download JSON'),
        'weight' => -5,
        'url' => Url::fromRoute('entity.site_scanner_result.download_json', ['site_scanner_result' => $entity->id()]),
      ];
    }

    if ($entity->access('delete')) {
      $operations['delete'] = [
        'title' => t('Delete'),
        'weight' => -1,
        'url' => Url::fromRoute('entity.site_scanner_result.delete_form', ['site_scanner_result' => $entity->id()]),
      ];
    }

    return $operations;
  }
}
